@@ -50,6 +50,7 @@ enum {
 
 struct bgState {
     GdkColor   bgColor1, bgColor2;
+    gint       enabled;
     gint       grad;
     gint       vertical;
     gint       bgType;
@@ -61,21 +62,24 @@ struct bgState {
 struct bgState origState, curState;
 
 /* The pointers to the color selectors */
-static GtkWidget *cp1, *cp2;
+static GtkWidget *cp1, *cp2, *cl2;
 
 void background_init(void);
 void background_properties_init(void);
 void background_read(struct bgState *state);
 void background_setup(struct bgState *state);
+static void background_write (struct bgState *state);
 void printState(struct bgState *state);
 void copyState(struct bgState *dest, struct bgState *src);
 void background_ok(GtkWidget *widget, struct bgState *state);
-
+Pixmap make_root_pixmap (gint width, gint height);
+void set_root_pixmap (Pixmap pixmap);
 
 void
 printState( struct bgState *state )
 {
     printf("\n-------------------------------\n");
+    printf("Background is : %s\n", (state->enabled) ? "Enabled" : "Disabled");
     printf("Color 1: #%04x%04x%04x\n",
 	      state->bgColor1.red >> 8,
 	      state->bgColor1.green >> 8,
@@ -167,7 +171,6 @@ fill_gradient (unsigned char *d, gint w, gint h,
 static void
 fill_monitor (int prop_changed, struct bgState *state)
 {
-    GdkWindow *rootWindow;
     ImlibImage *pi = NULL;
     ImlibImage *bi = NULL;
     GdkPixmap *screen;
@@ -197,10 +200,9 @@ fill_monitor (int prop_changed, struct bgState *state)
 	gdk_cursor_destroy (cursor);
     }
     
-    rootWindow = gdk_window_foreign_new (GDK_ROOT_WINDOW());
-    gdk_window_get_size (rootWindow, &rootWidth, &rootHeight);
+    gdk_window_get_size (GDK_ROOT_PARENT(), &rootWidth, &rootHeight);
     
-    if (state->bgType == BACKGROUND_WALLPAPER) {
+    if (state->bgType == BACKGROUND_WALLPAPER && state->enabled) {
 	bi = Imlib_load_image (imlib_data, state->wpFileName);
 	if (bi)
 	    Imlib_render (imlib_data, bi, 
@@ -211,39 +213,72 @@ fill_monitor (int prop_changed, struct bgState *state)
 	}
     }
     
-    if (capplet) {
-	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(cp1), &short_r, &short_g, &short_b, &short_a);
-    r = short_r;
-    g = short_g;
-    b = short_b;
-    a = short_a;
+    if (capplet && state->enabled) {
+	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(cp1), 
+				   &short_r, &short_g, &short_b, &short_a);
+	r = short_r; g = short_g;  b = short_b; a = short_a;
 
 	state->bgColor1.red = r;
 	state->bgColor1.green = g;
 	state->bgColor1.blue = b;
-	r >>= 8;
-	g >>= 8;
-	b >>= 8;
+	r >>= 8; g >>= 8; b >>= 8;
 	state->bgColor1.pixel = gdk_imlib_best_color_match(&r, &g, &b);
 
-	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(cp2), &short_r, &short_g, &short_b, &short_a);
-    r = short_r;
-    g = short_g;
-    b = short_b;
-    a = short_a;
+	gnome_color_picker_get_i16(GNOME_COLOR_PICKER(cp2), 
+				   &short_r, &short_g, &short_b, &short_a);
+	r = short_r; g = short_g; b = short_b; a = short_a;
     
 	state->bgColor2.red = r;
 	state->bgColor2.green = g;
 	state->bgColor2.blue = b;
-	r >>= 8;
-	g >>= 8;
-	b >>= 8;
+	r >>= 8; g >>= 8; b >>= 8;
 	state->bgColor2.pixel = gdk_imlib_best_color_match(&r, &g, &b);
     }
-    
+
+    if (!state->enabled) {
+	/* set bg black, put in text "Disabled" */
+	if (fillPreview) {
+	    GdkColor color;
+	    GdkFont  *font;
+	    gint     height, width;
+
+	    screen = GTK_PIXMAP (GTK_BIN (monitor)->child)->pixmap;
+	    cw = MONITOR_CONTENTS_WIDTH;
+	    ch = MONITOR_CONTENTS_HEIGHT;
+	    cx = MONITOR_CONTENTS_X;
+	    cy = MONITOR_CONTENTS_Y;
+	    if (!GTK_WIDGET_REALIZED (monitor))
+		gtk_widget_realize (monitor);
+	    gc = gdk_gc_new (monitor->window);
+
+	    gdk_color_black(gtk_widget_get_colormap(GTK_BIN(monitor)->child),
+			    &color);
+	    gdk_gc_set_foreground(gc, &color);
+	    gdk_draw_rectangle (screen, gc, TRUE, cx, cy, cw, ch);
+
+	    font = GTK_BIN(monitor)->child->style->font;
+	    width = gdk_string_width ( font, "Disabled");
+	    height = gdk_string_height ( font, "Disabled");
+
+	    gdk_color_white(gtk_widget_get_colormap(GTK_BIN(monitor)->child),
+			    &color);
+	    gdk_gc_set_foreground(gc, &color);
+	    gdk_draw_string (screen, font, gc, 
+			     cx + (cw-width)/2, cy + (ch-height)/2 + height/2,
+			     "Disabled");
+
+	    gdk_gc_unref (gc);
+	    gtk_widget_queue_draw (monitor);
+	}
+
+	if (capplet)
+	    gdk_window_set_cursor (capplet->window, NULL);
+	
+	return;
+    }
     /* are we working on root window ? */
     if (!fillPreview) {
-	gc = gdk_gc_new (rootWindow);
+	gc = gdk_gc_new (GDK_ROOT_PARENT());
 	xgc = ((GdkGCPrivate *) gc)->xgc;
 	
 	if ((!state->grad || (bi && !bi->shape_mask)) && 
@@ -253,11 +288,10 @@ fill_monitor (int prop_changed, struct bgState *state)
 
 		pix = Imlib_move_image (imlib_data, bi);
 		mask = Imlib_move_mask (imlib_data, bi);
-		bg = XCreatePixmap (GDK_DISPLAY (), GDK_ROOT_WINDOW (),
-				    bi->rgb_width, bi->rgb_height,
-				    gdkx_visual_get (Imlib_get_visual (imlib_data)->visualid)->depth);
+
+		bg = make_root_pixmap (bi->rgb_width, bi->rgb_height);
 		
-		gdk_color_alloc (gdk_window_get_colormap (rootWindow), 
+		gdk_color_alloc (gdk_window_get_colormap (GDK_ROOT_PARENT()), 
 				 &state->bgColor1);
 		gdk_gc_set_foreground (gc, &state->bgColor1);
 
@@ -276,20 +310,14 @@ fill_monitor (int prop_changed, struct bgState *state)
 			   bi->rgb_width, bi->rgb_height,
 			   0, 0);
 
-		XSetWindowBackgroundPixmap(GDK_DISPLAY(), GDK_ROOT_WINDOW(),
-					   bg);
-		
+		set_root_pixmap (bg);
+
 		Imlib_free_pixmap (imlib_data, pix);
 		Imlib_free_pixmap (imlib_data, mask);
 		Imlib_destroy_image (imlib_data, bi);
-		XClearWindow (GDK_DISPLAY (), GDK_ROOT_WINDOW ());
-		XFreePixmap (GDK_DISPLAY (), bg);
 	    } else {
-#ifdef DEBUG
-		g_print("Setting background in fill_monitor\n");
-#endif
-		gdk_window_set_background (rootWindow, &state->bgColor1);
-		gdk_window_clear (rootWindow);
+		gdk_window_set_background (GDK_ROOT_PARENT(), &state->bgColor1);
+		gdk_window_clear (GDK_ROOT_PARENT());
 	    }
 			
 	    gdk_gc_unref (gc);
@@ -315,10 +343,11 @@ fill_monitor (int prop_changed, struct bgState *state)
 	    cw = rootWidth;
 	    ch = rootHeight;
 	}
-/*	screen = gdk_pixmap_new (rootWindow, rootWidth, rootHeight, -1); */
-	screen = gdk_pixmap_new (rootWindow, cw, ch, -1); 
+	xscreen = make_root_pixmap (cw, ch);
     } else {
 	screen = GTK_PIXMAP (GTK_BIN (monitor)->child)->pixmap;
+	xscreen = GDK_WINDOW_XWINDOW (screen);
+
 	cw = MONITOR_CONTENTS_WIDTH;
 	ch = MONITOR_CONTENTS_HEIGHT;
 	cx = MONITOR_CONTENTS_X;
@@ -330,7 +359,6 @@ fill_monitor (int prop_changed, struct bgState *state)
 	gc = gdk_gc_new (monitor->window);
     }
     
-    xscreen = GDK_WINDOW_XWINDOW (screen);
     xgc = GDK_GC_XGC (gc);
     
     if (state->grad) {
@@ -362,7 +390,7 @@ fill_monitor (int prop_changed, struct bgState *state)
 	Imlib_free_pixmap (imlib_data, pix);
 	Imlib_destroy_image (imlib_data, pi);
     } else {
-	gdk_color_alloc (gdk_window_get_colormap (rootWindow), 
+	gdk_color_alloc (gdk_window_get_colormap (GDK_ROOT_PARENT()), 
 			 &state->bgColor1);
 	gdk_gc_set_foreground (gc, &state->bgColor1);
 	XFillRectangle (GDK_DISPLAY (), xscreen, xgc, cx, cy, cw, ch);
@@ -481,9 +509,7 @@ fill_monitor (int prop_changed, struct bgState *state)
     if (fillPreview) {
 	gtk_widget_queue_draw (monitor);
     } else {
-	gdk_window_set_back_pixmap (rootWindow, screen, FALSE);
-	gdk_pixmap_unref (screen);
-	gdk_window_clear (rootWindow);
+	set_root_pixmap (xscreen);
     }
     
     /* Reset the cursor to normal */
@@ -499,10 +525,6 @@ color_sel_set (GnomeColorPicker *gcp, gpointer data)
     if (!ignoreChanges)
 	capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
 
-#ifdef DEBUG
-    printf("Changing color, ignore changes is %d\n",ignoreChanges);
-    printState(&curState);
-#endif
 }
 
 static gint
@@ -512,37 +534,50 @@ idle_fill_monitor (gpointer data)
 	return FALSE;
 }
 
-static GtkWidget *radioh, *radiov, *radiof, *radiog;
+static GtkWidget *radioh, *radiov, *radiof, *radiog, *radion;
 static GtkWidget *tiledButton, *scaledButton;
+static GtkWidget *wallp, *fill;
 static GtkWidget *scaledkeepButton, *centeredButton;
 
 static void
 set_background_mode (GtkWidget *widget)
 {
     if (GTK_TOGGLE_BUTTON (widget)->active) {
-	if (widget == radiof) {
+        if (widget == radion) {
+	    /* Disabled */
+	    gtk_widget_set_sensitive(wallp, FALSE);
+	    gtk_widget_set_sensitive(fill, FALSE);
+	    curState.enabled = FALSE;
+        } else if (widget == radiof) {
 	    /* Flat color */
+	    gtk_widget_set_sensitive(cp1, TRUE);
 	    gtk_widget_set_sensitive(cp2, FALSE);
+	    gtk_widget_set_sensitive(cl2, FALSE);
 	    gtk_widget_set_sensitive(radioh, FALSE);
 	    gtk_widget_set_sensitive(radiov, FALSE);
+	    curState.enabled = TRUE;
 	    curState.grad = FALSE;
 	} else if (widget == radiog) {
 	    /* Gradient fill */
+	    gtk_widget_set_sensitive(cp1, TRUE);
 	    gtk_widget_set_sensitive(cp2, TRUE);
+	    gtk_widget_set_sensitive(cl2, TRUE);
 	    gtk_widget_set_sensitive(radioh, TRUE);
 	    gtk_widget_set_sensitive(radiov, TRUE);
+	    curState.enabled = TRUE;
 	    curState.grad = TRUE;
 	}
 
 	fill_monitor (!ignoreChanges, &curState);
-    }
+    } else if (widget == radion) {
+	    gtk_widget_set_sensitive(wallp, TRUE);
+	    gtk_widget_set_sensitive(fill, TRUE);
+	    curState.enabled = TRUE;
+	    fill_monitor (!ignoreChanges, &curState);
+    }	    
     if (!ignoreChanges)
 	capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
 
-#ifdef DEBUG
-    printf("Setting mode to %s\n",(widget == radiof) ? "Solid" : "Gradient");
-    printState(&curState);
-#endif
 }
 
 static void
@@ -559,10 +594,6 @@ set_orientation (GtkWidget *widget)
     if (!ignoreChanges)
 	capplet_widget_state_changed(CAPPLET_WIDGET(capplet), TRUE);
 
-#ifdef DEBUG
-    printf("Setting orientation to %s\n",(widget == radioh) ? "Horizontal" : "Vertical");
-    printState(&curState);
-#endif
 }
 
 static void
@@ -583,22 +614,17 @@ color_setup (struct bgState *state)
     GtkWidget *frame;
     GtkWidget *table;
     GtkWidget *button1;
-    GtkWidget *vb1, *vb2;
     
     frame = gtk_frame_new (_("Color"));
-    gtk_widget_show (frame);
     
-    vb1= gtk_vbox_new (FALSE, 0);
-    gtk_widget_show (vb1);
-    vb2= gtk_vbox_new (FALSE, 0);
-    gtk_widget_show (vb2);
     
-    table = gtk_table_new (2, 2, FALSE);
+    table = gtk_table_new (3, 5, FALSE);
     gtk_container_set_border_width (GTK_CONTAINER(table), GNOME_PAD);
     gtk_table_set_col_spacings (GTK_TABLE(table), GNOME_PAD);
     gtk_container_add (GTK_CONTAINER(frame), table);
-    gtk_widget_show (table);
     
+
+    /* The left side */
     cp1 = gnome_color_picker_new ();
     gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(cp1), 
 				state->bgColor1.red, state->bgColor1.green,
@@ -606,68 +632,63 @@ color_setup (struct bgState *state)
     gtk_signal_connect(GTK_OBJECT(cp1), "color_set", 
 		       GTK_SIGNAL_FUNC(color_sel_set), NULL);
 	
-    gtk_table_attach (GTK_TABLE(table), cp1, 0, 1, 0, 1, 0, 0, 0, 0);
-    gtk_widget_show (cp1);
-
-    radiof = gtk_radio_button_new_with_label (NULL, _("Flat"));
-    gtk_box_pack_start (GTK_BOX (vb1), radiof, FALSE, FALSE, 0);
-    gtk_table_attach_defaults (GTK_TABLE(table), vb1, 1, 2, 0, 1);
-    gtk_widget_show (radiof);
-    
-    radiog = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiof)),
-					      _("Gradient"));
-    gtk_box_pack_start (GTK_BOX (vb1), radiog, FALSE, FALSE, 0);
-    gtk_widget_show(radiog);
-
     cp2 = gnome_color_picker_new ();
     gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(cp2), 
 				state->bgColor2.red, state->bgColor2.green,
 				state->bgColor2.blue, 0xff);
     gtk_signal_connect(GTK_OBJECT(cp2), "color_set", 
 		       GTK_SIGNAL_FUNC(color_sel_set), NULL);
-    
-    gtk_table_attach (GTK_TABLE(table), cp2, 0, 1, 1, 2, 0, 0, 0, 0);
-    gtk_widget_set_sensitive (cp2, FALSE);
-    gtk_widget_show (cp2);
-    
-    radiov = gtk_radio_button_new_with_label (NULL, _("Vertical"));
-    gtk_box_pack_start (GTK_BOX (vb2), radiov, FALSE, FALSE, 0);
-    gtk_widget_set_sensitive (radiov, FALSE);
-    gtk_widget_show (radiov);
+    gtk_table_attach_defaults (GTK_TABLE(table), gtk_label_new (_("Color 1")), 0, 1, 0, 1);
+    gtk_table_attach (GTK_TABLE(table), cp1, 0, 1, 1, 2, 0, 0, 0, 0);
+    gtk_table_attach_defaults (GTK_TABLE(table), cp1, 0, 1, 1, 2);
+/*    gtk_table_attach_defaults (GTK_TABLE(table), gtk_hseparator_new (), 0, 1, 2, 3);*/
+    cl2 = gtk_label_new (_("Color 2"));
+    gtk_table_attach_defaults (GTK_TABLE(table), cl2, 0, 1, 3, 4);
+    gtk_table_attach (GTK_TABLE(table), cp2, 0, 1, 4, 5, 0, 0, 0, 0);
 
+    /* The middle side */
+    gtk_table_attach_defaults (GTK_TABLE(table), gtk_vseparator_new (), 1, 2, 0, 5);
+    
+    /* The right side */
+    radiof = gtk_radio_button_new_with_label (NULL, 
+					      _("Solid"));
+    radiog = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiof)),
+					      _("Gradient"));
+    radiov = gtk_radio_button_new_with_label (NULL, _("Vertical"));
     radioh = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(radiov)),
-						  _("Horizontal"));
-    gtk_box_pack_start (GTK_BOX (vb2), radioh, FALSE, FALSE, 0);
-    gtk_table_attach_defaults (GTK_TABLE(table), vb2, 1, 2, 1, 2);
-    gtk_widget_set_sensitive (radioh, FALSE);
-    gtk_widget_show(radioh);
+					      _("Horizontal"));
+    gtk_table_attach_defaults (GTK_TABLE(table), radiof, 2, 3, 0, 1);
+    gtk_table_attach_defaults (GTK_TABLE(table), radiog, 2, 3, 1, 2);
+    gtk_table_attach_defaults (GTK_TABLE(table), gtk_hseparator_new (), 2, 3, 2, 3);
+    gtk_table_attach_defaults (GTK_TABLE(table), radiov, 2, 3, 3, 4);
+    gtk_table_attach_defaults (GTK_TABLE(table), radioh, 2, 3, 4, 5);
 
     /* BEFORE our signals are connected, set initial states */
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON((state->grad) ? radiog : radiof), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->grad) ? radiog : radiof), TRUE);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->vertical) ? radiov : radioh), TRUE);
+
+    if (!state->grad) {
+	    gtk_widget_set_sensitive (cp2, FALSE);
+	    gtk_widget_set_sensitive (cl2, FALSE);
+	    gtk_widget_set_sensitive (radioh, FALSE);
+	    gtk_widget_set_sensitive (radiov, FALSE);
+    }
 
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON((state->vertical) ? radiov : radioh), TRUE);
-    
-    gtk_widget_set_sensitive(cp2, state->grad);
-    gtk_widget_set_sensitive(radioh, state->grad);
-    gtk_widget_set_sensitive(radiov, state->grad);
-	
     /* When people change the background mode controls, update */
     gtk_signal_connect (GTK_OBJECT(radiog), "toggled",
 			GTK_SIGNAL_FUNC(set_background_mode),
 			NULL);
     gtk_signal_connect (GTK_OBJECT(radiof), "toggled",
 			(GtkSignalFunc) set_background_mode,
-			NULL);
-    
+			NULL);    
     gtk_signal_connect (GTK_OBJECT(radiov), "toggled",
 			(GtkSignalFunc) set_orientation,
 			(gpointer) ((long) TRUE));
     gtk_signal_connect (GTK_OBJECT(radioh), "toggled",
 			(GtkSignalFunc) set_orientation,
 			(gpointer) ((long) FALSE));
-    
     gtk_idle_add ((GtkFunction) idle_fill_monitor, NULL);
-    
+    gtk_widget_show_all (frame);
     return frame;
 }
 
@@ -679,9 +700,6 @@ delete_browse (GtkWidget *w, GdkEvent *e, GtkWidget **f)
 	wpFileSelName = g_strdup (gtk_file_selection_get_filename
 				  (GTK_FILE_SELECTION (*f)));
 	*f = NULL;
-#ifdef DEBUG
-	g_print("delete_browse\n");
-#endif
 	return TRUE;
 }
 
@@ -693,7 +711,11 @@ browse_activated (GtkWidget *w, gchar *s)
 	curState.wpFileName = g_strdup(s);
 	curState.bgType = (s) ? BACKGROUND_WALLPAPER : BACKGROUND_SIMPLE;
 	/* printf ("%s\n", s); */
-
+/*
+	if (curState.bgType == BACKGROUND_WALLPAPER) {
+	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radiof), TRUE);
+	}
+*/
 	fill_monitor (TRUE, &curState);
 	/* printf ("%s\n", __FUNCTION__); */
 }
@@ -890,7 +912,7 @@ wallpaper_setup (struct bgState *state)
     rbut = gtk_radio_button_new_with_label
 	(gtk_radio_button_group (GTK_RADIO_BUTTON (rbut)),
 	 _("Scaled (keep aspect)"));
-    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (rbut),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rbut),
 				 (state->wpType == WALLPAPER_SCALED_KEEP));
     gtk_signal_connect (GTK_OBJECT (rbut), "toggled",
 			(GtkSignalFunc) set_wallpaper_type,
@@ -902,7 +924,7 @@ wallpaper_setup (struct bgState *state)
     rbut = gtk_radio_button_new_with_label
 	(gtk_radio_button_group (GTK_RADIO_BUTTON (rbut)),
 	 _("Centered"));
-    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (rbut),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rbut),
 				 (state->wpType == WALLPAPER_CENTERED));
     gtk_signal_connect (GTK_OBJECT (rbut), "toggled",
 			(GtkSignalFunc) set_wallpaper_type,
@@ -914,7 +936,7 @@ wallpaper_setup (struct bgState *state)
     rbut = gtk_radio_button_new_with_label
 	(gtk_radio_button_group (GTK_RADIO_BUTTON (rbut)),
 	 _("Tiled"));
-    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (rbut),
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (rbut),
 				 (state->wpType == WALLPAPER_TILED));
     gtk_signal_connect (GTK_OBJECT (rbut), "toggled",
 			(GtkSignalFunc) set_wallpaper_type,
@@ -950,14 +972,25 @@ background_apply (struct bgState *state)
     gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(cp2), 
 				state->bgColor2.red, state->bgColor2.green,
 				state->bgColor2.blue, 0xff);
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON((state->grad) ? radiog : radiof), TRUE);
+    if (!state->enabled)
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radion), TRUE);
+    else
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->grad) ? radiog : radiof), TRUE);
 
-    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON((state->vertical) ? radiov : radioh), TRUE);
-    
-    gtk_widget_set_sensitive(cp2, state->grad);
-    gtk_widget_set_sensitive(radioh, state->grad);
-    gtk_widget_set_sensitive(radiov, state->grad);
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON((state->vertical) ? radiov : radioh), TRUE);
 
+    if (state->enabled) {
+	gtk_widget_set_sensitive(cp2, state->grad);
+	gtk_widget_set_sensitive(cl2, state->grad);
+	gtk_widget_set_sensitive(radioh, state->grad);
+	gtk_widget_set_sensitive(radiov, state->grad);
+    } else {
+	gtk_widget_set_sensitive(cp1, FALSE);
+	gtk_widget_set_sensitive(cp2, FALSE);
+	gtk_widget_set_sensitive(cl2, FALSE);
+	gtk_widget_set_sensitive(radioh, FALSE);
+	gtk_widget_set_sensitive(radiov, FALSE);
+    }
     switch (state->wpType) {
       case WALLPAPER_SCALED:
 	choice = scaledButton;
@@ -974,11 +1007,10 @@ background_apply (struct bgState *state)
       default:
 	choice = tiledButton;
     }
-    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (choice), TRUE);
+    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (choice), TRUE);
 
     if (state->bgType!= BACKGROUND_SIMPLE) {
 	gchar *tmp;
-printf("setting filename back to %s\n",state->wpFileName);
 	/* ok this is horrible i should die */
 	tmp = g_strdup(state->wpFileName);
 	set_monitor_filename(tmp);
@@ -993,30 +1025,43 @@ printf("setting filename back to %s\n",state->wpFileName);
     ignoreChanges=FALSE;
 }
 
+static void
+background_help(GtkWidget *widget, struct bgState *state)
+{
+  gchar *tmp;
+
+  tmp = gnome_help_file_find_file ("users-guide", "gccdesktop.html#GCCBACK");
+  if (tmp) {
+    gnome_help_goto(0, tmp);
+    g_free(tmp);
+  }
+}
+
+
 static void
 background_try(GtkWidget *widget, struct bgState *state)
 {
+
+printState(state);
     background_apply(state);
 }
 
+static void
+background_cancel ()
+{
+    background_apply(&origState);
+    background_write(&origState);
+}
 static void
 background_revert ()
 {
     ignoreChanges = TRUE;
 
-#ifdef DEBUG
-    printf("Current State\n");
-    printState(&origState);
-#endif
 
     background_apply(&origState);
     fillPreview = TRUE;
     fill_monitor (FALSE, &origState);
 
-#ifdef DEBUG
-    printf("Restored State\n");
-    printState(&origState);
-#endif
 
     copyState(&curState, &origState);
     ignoreChanges = FALSE;
@@ -1028,10 +1073,6 @@ background_write (struct bgState *state)
     char buffer [60];
 
 
-#ifdef DEBUG
-    printf("Saving state to disk\n");
-    printState(state);
-#endif
     snprintf (buffer, sizeof(buffer), "#%02x%02x%02x",
 	      state->bgColor1.red >> 8,
 	      state->bgColor1.green >> 8,
@@ -1042,7 +1083,9 @@ background_write (struct bgState *state)
 	      state->bgColor2.green >> 8,
 	      state->bgColor2.blue >> 8);
     gnome_config_set_string ("/Background/Default/color2", buffer);
-    
+
+    gnome_config_set_string ("/Background/Default/Enabled",
+			     (state->enabled) ? "True" : "False");
     gnome_config_set_string ("/Background/Default/simple",
 			     (state->grad) ? "gradient" : "solid");
     gnome_config_set_string ("/Background/Default/gradient",
@@ -1081,6 +1124,10 @@ background_read ( struct bgState *state )
 	b = state->bgColor2.blue >> 8;
 	state->bgColor2.pixel = gdk_imlib_best_color_match(&r, &g, &b);
   
+	state->enabled = !(strcasecmp
+		(gnome_config_get_string
+		 ("/Background/Default/Enabled=True"),
+		 "True"));
 	state->bgType = (strcasecmp
 		  (gnome_config_get_string
 		   ("/Background/Default/type=simple"),
@@ -1122,9 +1169,6 @@ background_init() {
     background_setup(&origState);
     ignoreChanges = FALSE;
 
-#ifdef DEBUG
-    printState(&origState);
-#endif
 }
 
 void
@@ -1137,9 +1181,6 @@ background_properties_init() {
     fillPreview = TRUE;
     ignoreChanges = FALSE;
 
-#ifdef DEBUG
-    printState(&origState);
-#endif
 }
 
 /*
@@ -1172,14 +1213,13 @@ img_dnd_drop (GtkWidget *widget, GdkDragContext *context, gint x, gint y,
 void
 background_setup (struct bgState *state)
 {
-     static GtkTargetEntry drop_types [] = { 
+    static GtkTargetEntry drop_types [] = { 
 	     { "text/uri-list", 0, TARGET_URI_LIST },
-     };
-     static gint n_drop_types = sizeof (drop_types) / sizeof(drop_types[0]);
+    };
+    static gint n_drop_types = sizeof (drop_types) / sizeof(drop_types[0]);
 
     GtkWidget *settings;
     GtkWidget *vbox, *hbox;
-    GtkWidget *fill, *wallp;
     GtkWidget *align;
 
     capplet = capplet_widget_new();
@@ -1213,7 +1253,12 @@ background_setup (struct bgState *state)
     gtk_box_pack_start (GTK_BOX(hbox), align, TRUE, TRUE, 0);
 
     gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	
+
+    	
+    radion = gtk_check_button_new_with_label ( _("Disable background selection") );
+    gtk_widget_show (radion);
+    gtk_box_pack_end (GTK_BOX (vbox), radion, FALSE, FALSE, 0);
+
     settings = gtk_hbox_new (FALSE, GNOME_PAD);
     gtk_container_set_border_width (GTK_CONTAINER(settings), GNOME_PAD);
     gtk_box_pack_end (GTK_BOX (vbox), settings, TRUE, TRUE, 0);
@@ -1223,7 +1268,17 @@ background_setup (struct bgState *state)
 	
     wallp  = wallpaper_setup (state);
     gtk_box_pack_end (GTK_BOX (settings), wallp, TRUE, TRUE, 0);
-	
+
+    if (!state->enabled) {
+	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radion), TRUE);
+	    gtk_widget_set_sensitive (fill, FALSE);
+	    gtk_widget_set_sensitive (wallp, FALSE);
+    }
+
+    gtk_signal_connect (GTK_OBJECT(radion), "toggled",
+			(GtkSignalFunc) set_background_mode,
+			NULL);
+
     gtk_widget_show (align);
     gtk_widget_show (monitor);
     gtk_widget_show (settings);
@@ -1231,10 +1286,14 @@ background_setup (struct bgState *state)
     gtk_widget_show (vbox);
 
 
+    gtk_signal_connect (GTK_OBJECT (capplet), "help",
+			GTK_SIGNAL_FUNC (background_help), &curState);
     gtk_signal_connect (GTK_OBJECT (capplet), "try",
 			GTK_SIGNAL_FUNC (background_try), &curState);
     gtk_signal_connect (GTK_OBJECT (capplet), "revert",
 			GTK_SIGNAL_FUNC (background_revert), NULL);
+    gtk_signal_connect (GTK_OBJECT (capplet), "cancel",
+			GTK_SIGNAL_FUNC (background_cancel), NULL);
     gtk_signal_connect (GTK_OBJECT (capplet), "ok",
 			GTK_SIGNAL_FUNC (background_ok), &curState);
 
@@ -1254,6 +1313,73 @@ enum {
     ALIGN_KEY     = -8
 };
 
+/* Create a persistant pixmap. We create a separate display
+ * and set the closedown mode on it to RetainPermanent
+ */
+Pixmap
+make_root_pixmap (gint width, gint height)
+{
+	Display *display;
+	Pixmap result;
+	
+	display = XOpenDisplay (gdk_display_name);
+	XSetCloseDownMode (display, RetainPermanent);
+
+	result = XCreatePixmap (display,
+				DefaultRootWindow (display),
+				width, height,
+				gdkx_visual_get (Imlib_get_visual (imlib_data)->visualid)->depth);
+
+	XCloseDisplay (display);
+
+	return result;
+}
+
+/* Set the root pixmap, and properties pointing to it. We
+ * do this atomically with XGrabServer to make sure that
+ * we won't leak the pixmap if somebody else it setting
+ * it at the same time. (This assumes that they follow the
+ * same conventions we do
+ */
+void 
+set_root_pixmap (Pixmap pixmap) {
+	GdkAtom type;
+	gulong nitems, bytes_after;
+	gint format;
+	guchar *data_esetroot;
+
+	XGrabServer (GDK_DISPLAY());
+
+	XGetWindowProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+			    gdk_atom_intern("ESETROOT_PMAP_ID", FALSE),
+			    0L, 1L, False, XA_PIXMAP,
+			    &type, &format, &nitems, &bytes_after,
+			    &data_esetroot);
+
+	if (type == XA_PIXMAP) {
+		if (format == 32 && nitems == 4)
+			XKillClient(GDK_DISPLAY(), *((Pixmap*)data_esetroot));
+
+		XFree (data_esetroot);
+	}
+
+	XChangeProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+			 gdk_atom_intern("ESETROOT_PMAP_ID", FALSE), XA_PIXMAP,
+			 32, PropModeReplace,
+			 (guchar *) &pixmap, 1);
+	XChangeProperty (GDK_DISPLAY(), GDK_ROOT_WINDOW(),
+			 gdk_atom_intern("_XROOTPMAP_ID", FALSE), XA_PIXMAP,
+			 32, PropModeReplace,
+			 (guchar *) &pixmap, 1);
+
+	XSetWindowBackgroundPixmap (GDK_DISPLAY(), GDK_ROOT_WINDOW(), pixmap);
+	XClearWindow (GDK_DISPLAY (), GDK_ROOT_WINDOW ());
+
+	XUngrabServer (GDK_DISPLAY());
+	
+	XFlush(GDK_DISPLAY());
+}
+
 #if 0
 /* Options used by background properties.  */
 static struct argp_option arguments[] =
