@@ -37,6 +37,7 @@ EditorDescription possible_editors[] =
         { "vim",  "vim", TRUE, "executable", TRUE },
         { "gvim",  "gvim", FALSE, "executable", TRUE },
         { "ed", "ed", TRUE, "executable", FALSE },
+        { "GMC/CoolEdit", "gmc -e", FALSE, "mc-internal", FALSE },
 };
 
 static GtkWidget *capplet;
@@ -92,6 +93,18 @@ edit_read(void)
         original_info.accepts_lineno = gnome_config_get_bool("/editor/Editor/ACCEPTS_LINE_NO=TRUE");
 }
 
+static void
+edit_help(void)
+{
+  gchar *tmp;
+
+  tmp = gnome_help_file_find_file ("users-guide", "gccedit.html");
+  if (tmp) {
+    gnome_help_goto(0, tmp);
+    g_free(tmp);
+  }
+}
+
 static void
 edit_apply(void)
 {
@@ -151,6 +164,8 @@ edit_setup(void)
         
 	/* Finished */
 
+        gtk_signal_connect (GTK_OBJECT (capplet), "help",
+                            GTK_SIGNAL_FUNC (edit_help), NULL);
         gtk_signal_connect (GTK_OBJECT (capplet), "try",
                             GTK_SIGNAL_FUNC (edit_apply), NULL);
         gtk_signal_connect (GTK_OBJECT (capplet), "revert",
@@ -186,6 +201,7 @@ main (int argc, char **argv)
 
         edit_setup ();
         edit_read ();
+        set_combo (original_info.executable_name);
         capplet_gtk_main ();
 
 	return 0;
