@@ -1,5 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
-/* Copyright (C) 1998 Redhat Software Inc. 
+/* Copyright (C) 1998 Redhat Software Inc.
+ * Code available under the Gnu GPL.
  * Authors: Jonathan Blandford <jrb@redhat.com>
  */
 #include <config.h>
@@ -32,6 +33,7 @@ gboolean dpms;
 gboolean usessaver;
 gboolean password;
 gchar *screensaver;
+gchar *random_screensaver = "RANDOM SCREENSAVER";
 screensaver_data *sd;
 
 /* saving info */
@@ -40,7 +42,6 @@ get_saver_frame ()
 {
         GtkWidget *retval;
         GtkWidget *vbox;
-        GtkWidget *swindow;
         GtkWidget *temphbox;
 
         retval = gtk_frame_new (_("Screen Saver"));
@@ -53,11 +54,7 @@ get_saver_frame ()
         setup_label = gtk_label_new (_("Settings..."));
         setup_button = gtk_button_new ();
 
-	swindow = gtk_scrolled_window_new (NULL, NULL);
-        gtk_widget_set_usize (swindow, 150, -1 );
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (swindow), get_and_set_mode());
-        gtk_box_pack_start (GTK_BOX (vbox), swindow, TRUE, TRUE, 0);
+        gtk_box_pack_start (GTK_BOX (vbox), get_and_set_mode (), TRUE, TRUE, 0);
 
         gnome_widget_add_help (setup_button, _("Pressing this button will popup a dialog"\
                                                "box that will help you setup the current screensaver."));
@@ -155,7 +152,6 @@ get_demo_frame()
 
         monitor = gtk_drawing_area_new ();
         gtk_widget_set_usize (monitor, 200, 150);
-        
 	gtk_container_set_border_width (GTK_CONTAINER (inner_frame), GNOME_PAD_SMALL);
         gtk_container_add (GTK_CONTAINER (retval), inner_frame);
         gtk_container_add (GTK_CONTAINER (inner_frame), monitor);
@@ -194,7 +190,7 @@ main (int argc, char **argv)
 {
         GnomeClient *client = NULL;
         GnomeClientFlags flags;
-        gchar *session_args[2];
+        gchar *session_args[3];
         gint token, init_results;
 
         bindtextdomain (PACKAGE, GNOMELOCALEDIR);
@@ -232,27 +228,34 @@ main (int argc, char **argv)
 
                 gnome_client_flush (client);
         }
-	else
+        else
 		token = 1;
 
         if(token) {
                 gchar *command, *r_command;
-                gint pid;
+                gchar dpmstime[11];
+                gint mypid;
+                command = gnome_config_get_string ("/Screensaver/Default/command=xscreensaver -no-splash -timeout 20 -nice 10 -lock-mode");
+                r_command = g_strconcat (command, " &", NULL);
+                system ("xscreensaver-command -exit");
 
-                command = gnome_config_get_string ("/Screensaver/Default/command=xscreensaver -no-splash");
-                r_command = g_copy_strings (command, " >/dev/null 2>&1", NULL);
-                g_print ("Screen saver command: %s\n",r_command);
-                pid = gnome_execute_shell(getenv("HOME"),
-                                          "xscreensaver-command"
-                                          " -exit >/dev/null 2>&1");
-                waitpid(pid, &pid, 0);
-
-                gnome_execute_shell(getenv("HOME"), r_command);
+                system (r_command);
                 g_free (command);
                 g_free (r_command);
+                
+                /* and now, a shameful reuse of variables to handle dpm. (: */
+                mypid = 0;
+                mypid = gnome_config_get_bool ("/Screensaver/Default/dpms=true");
+                if (mypid) {
+                        mypid = gnome_config_get_string ("/Screensaver/Default/dpmsmins=20");
+                        mypid += gnome_config_get_string ("/Screensaver/Default/waitmins=20");
+                        g_snprintf (dpmstime, 10, "%d",(gint) mypid * 60);
+                        r_command = g_strconcat ("xset dpms ", dpmsmins, " ", dpmsmins, " ", dpmsmins, NULL);
+                        system (r_command);
+                        g_free (r_command);
+                }
         }
-
-	if (init_results != 1) {
+	if (init_results == 0) {
                 screensaver_load ();
                 screensaver_setup();
                 signal (SIGCHLD, sig_child);
@@ -266,9 +269,15 @@ main (int argc, char **argv)
                                     GTK_SIGNAL_FUNC (revert_callback), NULL);
                 gtk_signal_connect (GTK_OBJECT (capplet), "ok", 
                                     GTK_SIGNAL_FUNC (ok_callback), NULL);
+                gtk_signal_connect (GTK_OBJECT (capplet), "help", 
+                                    GTK_SIGNAL_FUNC (help_callback), NULL);
+                gtk_signal_connect (GTK_OBJECT (capplet), "page_hidden", 
+                                    GTK_SIGNAL_FUNC (page_hide_callback), NULL);
+                gtk_signal_connect (GTK_OBJECT (capplet), "page_shown", 
+                                    GTK_SIGNAL_FUNC (page_show_callback), NULL);
                 gtk_signal_connect (GTK_OBJECT (monitor), "expose_event", 
-                                    GTK_SIGNAL_FUNC (list_expose_callback),
-                                    sd);  /* disabled for now */
+                                    GTK_SIGNAL_FUNC (monitor_expose_callback),
+                                    sd); 
                 sd = NULL;
 
 	        capplet_gtk_main ();
