@@ -26,6 +26,8 @@ typedef struct {
     GtkWidget *btn_filename, *btn_play;
 
     gint ignore_changed;
+
+    GHashTable *by_category;
 } SoundProps;
 
 static SoundProps *sound_properties_create(void);
@@ -54,10 +56,13 @@ static void sound_properties_event_change_file(GtkEditable *entry,
                                                SoundProps *props);
 static void sound_properties_set_sensitivity(GtkToggleButton *btn,
                                              SoundProps *props);
-static void sound_properties_event_apply(GtkCTreeNode *node, SoundProps *props); 
+static void sound_properties_event_apply(GtkCTree *ctree,
+                                         GtkCTreeNode *node,
+                                         SoundProps *props);
 static void sound_properties_apply(SoundProps *props); 
 static void ui_do_revert(GtkWidget *w, SoundProps *props);
 static void ui_do_ok(GtkWidget *w, SoundProps *props);
+static void ui_do_help(void);
 static void ui_do_cancel(GtkWidget *w, SoundProps *props);
 static void sound_properties_play_sound(GtkWidget *btn, SoundProps *props); 
 static void reload_esd_samples(const char *config_path);
@@ -69,10 +74,13 @@ main(int argc, char *argv[])
 {
     GnomeClient *client;
     GnomeClientFlags flags;
-    gchar *session_args[2];
+    gchar *session_args[3];
     int token, init_results;
     SoundProps *sound_properties;
 
+    bindtextdomain (PACKAGE, GNOMELOCALEDIR);
+    textdomain (PACKAGE);
+
     init_results = gnome_capplet_init("sound-properties", VERSION,
                                       argc, argv, NULL, 0, NULL);
 
@@ -87,7 +95,7 @@ main(int argc, char *argv[])
     else
         flags = 0;
 
-	if (flags & GNOME_CLIENT_IS_CONNECTED) {
+    if (flags & GNOME_CLIENT_IS_CONNECTED) {
 #ifdef HAVE_ESD
 		token = gnome_startup_acquire_token("GNOME_SOUND_PROPERTIES",
                                             gnome_client_get_id(client));
@@ -111,6 +119,13 @@ main(int argc, char *argv[])
     } else
 		token = 1;
 
+#if 0
+    {
+        volatile int x=1;
+        while(x);
+    }
+#endif
+
     switch(init_results) {
     case 1:
 #ifdef HAVE_ESD
@@ -118,16 +133,24 @@ main(int argc, char *argv[])
             if(gnome_sound_connection < 0
                && gnome_config_get_bool("/sound/system/settings/start_esd=true")) {
                 int esdpid;
-                static const char *esd_cmdline[] = {"esd", NULL};
+                static const char *esd_cmdline[] = {"esd", "-nobeeps", NULL};
                 char *tmpargv[3];
                 char argbuf[32];
-                
-                esdpid = gnome_execute_async(NULL, 1, (char **)esd_cmdline);
+                time_t starttime;
+
+                esdpid = gnome_execute_async(NULL, 2, (char **)esd_cmdline);
                 g_snprintf(argbuf, sizeof(argbuf), "%d", esdpid);
                 tmpargv[0] = "kill"; tmpargv[1] = argbuf; tmpargv[2] = NULL;
                 gnome_client_set_shutdown_command(client, 2, tmpargv);
-                sleep(1); /* lame... */
+		starttime = time(NULL);
                 gnome_sound_init(NULL);
+		while(gnome_sound_connection < 0
+		      && ((time(NULL) - starttime) < 4)) {
+#ifdef HAVE_USLEEP
+			usleep(1000);
+#endif
+	                gnome_sound_init(NULL);
+		}
             }
             
             if(gnome_sound_connection >= 0
@@ -168,6 +191,8 @@ sound_properties_create(void)
 
     retval = g_new0(SoundProps, 1);
 
+    retval->by_category = g_hash_table_new(g_str_hash, g_str_equal);
+
 #ifdef TESTING
     retval->capplet = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_window_set_policy(GTK_WINDOW(retval->capplet), TRUE, TRUE, TRUE);
@@ -178,6 +203,8 @@ sound_properties_create(void)
 #endif
 
 #ifdef HAVE_ESD
+    gtk_signal_connect(GTK_OBJECT(retval->capplet), "help",
+                       ui_do_help, NULL);
     gtk_signal_connect(GTK_OBJECT(retval->capplet), "revert",
                        ui_do_revert, retval);
     gtk_signal_connect(GTK_OBJECT(retval->capplet), "ok",
@@ -205,7 +232,7 @@ sound_properties_create(void)
             g_free (filename);
     }   
 
-    frame = gtk_frame_new ("Enable");
+    frame = gtk_frame_new (_("Enable"));
     wtmp = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
     gtk_container_add (GTK_CONTAINER (frame), wtmp);
 
@@ -246,12 +273,21 @@ sound_properties_create(void)
     /* * * * page "Sound Events" * * * */
     retval->table = table = gtk_table_new(2, 2, FALSE);
     gtk_table_set_row_spacings (GTK_TABLE (table), GNOME_PAD);
+
+    gtk_table_attach(GTK_TABLE(table),
+                     (retval->btn_filename = gnome_file_entry_new(NULL, _("Select sound file"))),
+                     1, 2, 1, 2,
+                     GTK_EXPAND | GTK_SHRINK, 0,
+                     GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+    path = gnome_datadir_file ("sounds/");
+    gnome_file_entry_set_default_path(GNOME_FILE_ENTRY (retval->btn_filename), path);
+    g_free (path);
     
-    /* FIXME: This needs to create the columns one at a time to enable i18n. */
+    ctree_column_titles[0] = _(ctree_column_titles[0]);
+    ctree_column_titles[1] = _(ctree_column_titles[1]);
+    ctree_column_titles[2] = _(ctree_column_titles[2]);
     retval->ctree = gtk_ctree_new_with_titles(3, 0,
                                               (gchar **)ctree_column_titles);
-    gtk_signal_connect(GTK_OBJECT(retval->ctree), "tree_select_row",
-                       sound_properties_event_select, retval);
 
     gtk_clist_set_selection_mode(GTK_CLIST(retval->ctree), GTK_SELECTION_BROWSE);
 
@@ -261,19 +297,6 @@ sound_properties_create(void)
 
     sound_properties_regenerate_ctree(retval);
 
-    /* Set size _after_ putting contents in */
-#if 0
-    gtk_clist_set_column_auto_resize(GTK_CLIST(retval->ctree), 0, TRUE);
-    gtk_clist_set_column_auto_resize(GTK_CLIST(retval->ctree), 1, TRUE);
-    gtk_clist_set_column_auto_resize(GTK_CLIST(retval->ctree), 2, TRUE);
-    gtk_clist_set_column_width(GTK_CLIST(retval->ctree), 0, 150);
-    gtk_clist_set_column_width(GTK_CLIST(retval->ctree), 1, 200);
-    gtk_clist_set_column_width(GTK_CLIST(retval->ctree), 2, 200);
-    gtk_widget_set_usize(GTK_WIDGET(wtmp), 150+200+200+20,
-                         250);
-
-#endif
-
     gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
     wtmp = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_set_usize(GTK_WIDGET(wtmp),
@@ -289,20 +312,7 @@ sound_properties_create(void)
                      0, 1, 1, 2,
                      GTK_EXPAND|GTK_SHRINK, 0,
                      GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-    gtk_signal_connect(GTK_OBJECT(retval->btn_play), "clicked", sound_properties_play_sound, retval);
-
-    gtk_table_attach(GTK_TABLE(table),
-                     (retval->btn_filename = gnome_file_entry_new(NULL, _("Select sound file"))),
-                     1, 2, 1, 2,
-                     GTK_EXPAND | GTK_SHRINK, 0,
-                     GNOME_PAD_SMALL, GNOME_PAD_SMALL);
-    path = gnome_datadir_file ("sounds/");
-    gnome_file_entry_set_default_path(GNOME_FILE_ENTRY (retval->btn_filename), path);
-    g_free (path);
-    gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(retval->btn_filename))),
-                       "changed",
-                       GTK_SIGNAL_FUNC(sound_properties_event_change_file),
-                       retval);
+    gtk_object_set_data(GTK_OBJECT(retval->btn_play), "gnome_disable_sound_events", GINT_TO_POINTER(1));
 
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table,
                              gtk_label_new(_("Sound Events")));
@@ -312,10 +322,20 @@ sound_properties_create(void)
     gtk_container_add(GTK_CONTAINER(retval->capplet), notebook);
 
     sound_properties_set_sensitivity(NULL, retval);
+
+    gtk_signal_connect(GTK_OBJECT(retval->ctree), "tree_select_row",
+                       sound_properties_event_select, retval);
+    gtk_signal_connect(GTK_OBJECT(retval->btn_play), "clicked",
+                       sound_properties_play_sound, retval);
+    gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(retval->btn_filename))),
+                       "changed",
+                       GTK_SIGNAL_FUNC(sound_properties_event_change_file),
+                       retval);
 #else
-    gtk_container_add(GTK_CONTAINER(retval->capplet), gtk_label_new("This copy of the GNOME control center was not compiled with sound support"));
+    gtk_container_add(GTK_CONTAINER(retval->capplet), gtk_label_new(_("This copy of the GNOME control center was not compiled with sound support")));
 #endif
 
+
     gtk_widget_show_all(retval->capplet);
 
     return retval;
@@ -373,9 +393,11 @@ sound_properties_read_path(SoundProps *props,
     SoundEvent *new_entry;
     char *category_name, *sample_name, *sample_file, *ctmp;
     gpointer top_iter, event_iter;
-    GtkCTreeNode *category_node = NULL, *event_node = NULL;
+    GtkCTreeNode *category_node = NULL, *event_node = NULL,
+        *prev_event_node = NULL;
     char *arow[4] = {NULL,NULL,NULL,NULL};
     struct dirent *dent;
+    GHashTable *cathash;
 
     dirh = opendir(path);
     if(!dirh)
@@ -403,50 +425,98 @@ sound_properties_read_path(SoundProps *props,
             }
         }
 
-        category_node = gtk_ctree_insert_node(GTK_CTREE(props->ctree),
-                                              NULL, NULL,
-                                              arow, GNOME_PAD_SMALL,
-                                              NULL, NULL, NULL, NULL, FALSE,
-                                              FALSE);
-        gtk_ctree_node_set_selectable(GTK_CTREE(props->ctree), category_node,
-                                      FALSE);
+        category_node = g_hash_table_lookup(props->by_category,
+                                            dent->d_name);
+
+        if(!category_node) {
+            category_node = gtk_ctree_insert_node(GTK_CTREE(props->ctree),
+                                                  NULL, NULL,
+                                                  arow, GNOME_PAD_SMALL,
+                                                  NULL, NULL, NULL, NULL, FALSE,
+                                                  FALSE);
+            gtk_ctree_node_set_selectable(GTK_CTREE(props->ctree), category_node,
+                                          FALSE);
+            gtk_ctree_node_set_row_data(GTK_CTREE(props->ctree),
+                                        category_node,
+                                        g_hash_table_new(g_str_hash,
+                                                         g_str_equal));
+            g_hash_table_insert(props->by_category, g_strdup(dent->d_name),
+                                category_node);
+        }
+
+        cathash = gtk_ctree_node_get_row_data(GTK_CTREE(props->ctree),
+                                              category_node);
 
         g_free(arow[0]);
 
-        event_node = NULL;
+        event_node = prev_event_node = NULL;
         event_iter = gnome_config_init_iterator_sections(tmpstr->str);
         while((event_iter = gnome_config_iterator_next(event_iter,
                                                        &sample_name,
                                                        NULL))) {
+
             if(!strcmp(sample_name, "__section_info__")) {
                 g_free(sample_name);
                 continue;
             }
 
-            arow[0] = NULL;
-            g_string_sprintf(tmpstr, "%s/description", sample_name);
-            arow[1] = gnome_config_get_translated_string(tmpstr->str);
-            if(!arow[1] || !*arow[1]) {
-                g_free(arow[1]);
-                arow[1] = g_strdup(sample_name);
-            }
             g_string_sprintf(tmpstr, "%s/file", sample_name);
             arow[2] = sample_file = gnome_config_get_string(tmpstr->str);
 
-            event_node = gtk_ctree_insert_node(GTK_CTREE(props->ctree),
-                                               category_node, event_node,
-                                               arow, GNOME_PAD_SMALL,
-                                               NULL, NULL, NULL, NULL, TRUE,
-                                               FALSE);
-            g_free(arow[1]);
-            new_entry = g_new0(SoundEvent, 1);
-            new_entry->category = g_strdup(dent->d_name);
-            new_entry->name = sample_name;
-            new_entry->file = sample_file;
-            new_entry->row = event_node;
-            gtk_ctree_node_set_row_data_full(GTK_CTREE(props->ctree), event_node,
-                                             new_entry,
-                                             (GtkDestroyNotify)sound_properties_event_free);
+            event_node = g_hash_table_lookup(cathash, sample_name);
+
+            if(event_node) {
+
+                new_entry = gtk_ctree_node_get_row_data(GTK_CTREE(props->ctree),
+                                                        event_node);
+#if 0
+                g_print("Getting entry %p node %p: %s\n", new_entry,
+                        event_node, sample_name);
+#endif
+
+                g_free(new_entry->file);
+
+                gtk_ctree_node_set_text(GTK_CTREE(props->ctree), event_node,
+                                        2, sample_file);
+
+                new_entry->file = sample_file;
+                g_free(sample_name);
+
+            } else {
+
+                arow[0] = NULL;
+                g_string_sprintf(tmpstr, "%s/description", sample_name);
+                arow[1] = gnome_config_get_translated_string(tmpstr->str);
+                if(!arow[1] || !*arow[1]) {
+                    g_free(arow[1]);
+                    arow[1] = g_strdup(sample_name);
+                }
+
+                event_node = gtk_ctree_insert_node(GTK_CTREE(props->ctree),
+                                                   category_node, prev_event_node,
+                                                   arow, GNOME_PAD_SMALL,
+                                                   NULL, NULL, NULL, NULL, TRUE,
+                                                   FALSE);
+                g_free(arow[1]);
+                new_entry = g_new0(SoundEvent, 1);
+                new_entry->category = g_strdup(dent->d_name);
+                new_entry->name = sample_name;
+                new_entry->file = sample_file;
+                new_entry->row = event_node;
+
+                gtk_ctree_node_set_row_data_full(GTK_CTREE(props->ctree),
+                                                 event_node,
+                                                 new_entry,
+                                                 (GtkDestroyNotify)sound_properties_event_free);
+                g_hash_table_insert(cathash, new_entry->name, event_node);
+
+#if 0
+                g_print("Adding entry %p node %p: %s\n", new_entry,
+                        event_node, sample_name);
+#endif
+
+                prev_event_node = event_node;
+            }
         }
 
         gnome_config_pop_prefix();
@@ -497,28 +567,32 @@ sound_properties_event_select(GtkCTree *ctree,
     props->ignore_changed--;
 }
 
-
 static void
-sound_properties_event_apply(GtkCTreeNode *node,
+sound_properties_event_apply(GtkCTree *ctree,
+                             GtkCTreeNode *node,
                              SoundProps *props)
 {
     SoundEvent *ev;
-    char *cur_filename, *ctmp;
+    char *cur_filename = NULL, *ctmp;
     GtkWidget *entry;
 
-    ev = gtk_ctree_node_get_row_data(GTK_CTREE(props->ctree), node);
+    ev = gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node);
+#if 0
+    g_print("get row data on %p = %p", node, ev);
+#endif
 
-    if(!ev)
-        return;
+    g_return_if_fail(ev);
 
-    gtk_ctree_node_get_text(GTK_CTREE(props->ctree),
+    gtk_ctree_node_get_text(GTK_CTREE(ctree),
                             node, 2, &cur_filename);
 
+    if(!cur_filename) return;
+
     /* If the user didn't change the setting, no need to set it */
     if(!strcmp(cur_filename, ev->file))
         return;
 
-    ctmp = g_strconcat("/sound/events/", ev->category, "/", ev->name, "/file");
+    ctmp = g_strconcat("/sound/events/", ev->category, "/", ev->name, "/file", NULL);
     gnome_config_set_string(ctmp, cur_filename);
     g_free(ctmp);
 }
@@ -526,8 +600,8 @@ sound_properties_event_apply(GtkCTreeNode *node,
 static void
 sound_properties_apply(SoundProps *props)
 {
-    g_list_foreach(GTK_CLIST(props->ctree)->row_list,
-                   (GFunc)sound_properties_event_apply, props);
+    gtk_ctree_post_recursive(GTK_CTREE(props->ctree), NULL,
+                             (GtkCTreeFunc)sound_properties_event_apply, props);
 
     gnome_config_set_bool("/sound/system/settings/start_esd",
                           GTK_TOGGLE_BUTTON(props->enable_esd_startup)->active);
@@ -548,8 +622,7 @@ sound_properties_set_sensitivity(GtkToggleButton *btn,
                              GTK_TOGGLE_BUTTON(props->enable_esd_startup)->active);
 
     page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(props->notebook), 1);
-    do_events = GTK_TOGGLE_BUTTON(props->enable_esd_startup)->active
-        && GTK_TOGGLE_BUTTON(props->enable_sound_events)->active;
+    do_events = GTK_TOGGLE_BUTTON(props->enable_sound_events)->active;
 
     gtk_widget_set_sensitive(page, do_events);
 
@@ -571,6 +644,18 @@ ui_do_revert(GtkWidget *w, SoundProps *props)
     sound_properties_regenerate_ctree(props);
 }
 
+static void
+ui_do_help(void)
+{
+  gchar *tmp;
+
+  tmp = gnome_help_file_find_file ("users-guide", "gccmm.html#GCCSOUND");
+  if (tmp) {
+    gnome_help_goto(0, tmp);
+    g_free(tmp);
+  }
+}
+
 static void
 ui_do_ok(GtkWidget *w, SoundProps *props)
 {
@@ -595,12 +680,27 @@ sound_properties_play_sound(GtkWidget *btn, SoundProps *props)
     node = GTK_CTREE_NODE(GTK_CLIST(props->ctree)->selection->data);
     ctmp = GTK_CLIST_ROW(&node->list)->cell[2].u.text;
 
-    if(*ctmp == '/')
+    if(*ctmp == '/' || g_file_exists(ctmp))
         ctmp2 = g_strdup(ctmp);
     else
         ctmp2 = gnome_sound_file(ctmp);
 
-    gnome_sound_play(ctmp2);
+    if(ctmp2 && g_file_exists(ctmp2))
+       gnome_sound_play(ctmp2);
+    else {
+        GnomeDialog *dlg;
+        char *msg;
+
+        if(ctmp2 && ctmp2[0]=='/')
+            msg = _("The sound file for this event does not exist.");
+        else
+            msg = _("The sound file for this event does not exist. You may want to install the gnome-audio package for a set of default sounds.");
+
+        dlg = GNOME_DIALOG(gnome_message_box_new(msg,
+                                                 GNOME_MESSAGE_BOX_ERROR,
+                                                 _("Close"), NULL));
+        gnome_dialog_run(dlg);
+    }
 
     g_free(ctmp2);
 }
