@@ -13,9 +13,13 @@ main(int argc, char **argv)
 
   set_tmp_rc();
   do_demo(argc, argv);
-  gnome_capplet_init ("theme-switcher-capplet",
-		      THEME_SWITCHER_VERSION, argc, argv, NULL, 0, NULL);
-  
+  switch (gnome_capplet_init ("theme-switcher-capplet",
+			      THEME_SWITCHER_VERSION, argc, argv, NULL, 0, NULL)) {
+  case -1:
+	  exit (1);
+  case 1:
+	  return 0;
+  }  
   w = make_main();
   gtk_widget_show_all(w);
   send_socket();
