@@ -13,6 +13,7 @@ extern GtkWidget *exit_dialog;
 extern GtkWidget *notebook;
 extern GtkWidget *create_exit_dialog();
 extern GList *capplet_list;
+static GtkWidget *about = NULL;
 
 /* this is meant to be called with a foreach to make a list of all modified nodes. */
 void
@@ -51,21 +52,37 @@ exit_dialog_cancel_callback(GtkWidget *widget, gpointer data)
         gnome_dialog_close (GNOME_DIALOG (exit_dialog));
         exit_dialog = NULL;
 }
-
 void
-about_callback(GtkWidget *widget, gpointer data)
+help_callback (GtkWidget *widget, gpointer data)
+{
+        gchar *tmp;
+        tmp = gnome_help_file_find_file ("users-guide", "gccdesktop.html");
+        if (tmp) {
+                gnome_help_goto(0, tmp);
+                g_free(tmp);
+        }
+}
+static void
+delete_callback (GtkWidget *widget, gpointer data)
+{
+        about = NULL;
+}
+void
+about_callback (GtkWidget *widget, gpointer data)
 {
-	GtkWidget *about = NULL;
 	gchar *authors[] = {
                 "Jonathan Blandford <jrb@redhat.com>",
 		NULL
 	};
 
-	about = gnome_about_new(_("Desktop Manager"), "0.1",
-				"Copyright (C) 1998 the Free Software Foundation",
-				(const gchar **) authors,
-				_("Desktop Properties manager."),
-				NULL);
+        if (about == NULL) {
+                about = gnome_about_new(_("GNOME Control Center"), VERSION,
+                                        "Copyright (C) 1999 Red Hat Software Inc.",
+                                        (const gchar **) authors,
+                                        _("Desktop Properties manager."),
+                                        NULL);
+                gtk_signal_connect (GTK_OBJECT (about), "destroy", delete_callback, NULL);
+        }
         gtk_widget_show(about);
 }
 void
@@ -78,6 +95,6 @@ exit_row_callback(GtkWidget *widget, gint row, gint column, GdkEventButton * eve
                 data = (node_data *) gtk_clist_get_row_data (GTK_CLIST (widget),row);
                 gnome_dialog_close (GNOME_DIALOG (exit_dialog));
                 exit_dialog = NULL;
-                launch_capplet (data);
+                launch_capplet (data, TRUE);
         }
 }
