@@ -9,8 +9,8 @@
 /* variables */
 extern GtkWidget *container;
 extern GtkWidget *notebook;
-extern GtkWidget *main_window;
 extern GtkWidget *splash_screen;
+extern GtkWidget *hack_widget;
 extern gchar *ior;
 static gint current_page = 0;
 static gint id = 0;
@@ -26,15 +26,25 @@ struct _capplet_entry
         CORBA_Object *cco;
         gboolean alive;
 };
-
+typedef struct _timer_entry timer_entry;
+struct _timer_entry
+{
+        /* we don't use nd->socket; b/c it can change, and we want one per timer_entry */
+        GtkWidget *socket;
+        gint count;
+        guint timer_id;
+};
 /* prototypes */
 void try_button_callback(GtkWidget *widget, gpointer data);
 void revert_button_callback(GtkWidget *widget, gpointer data);
 void ok_button_callback(GtkWidget *widget, gpointer data);
 void cancel_button_callback(GtkWidget *widget, gpointer data);
 void help_button_callback(GtkWidget *widget, gpointer data);
-void close_capplet (GtkWidget *widget, gpointer data);
+void close_capplet (gboolean show_splash, gpointer data);
 static void exec_capplet (node_data *data);
+static void shutdown_capplet (node_data *nd);
+static void capplet_close_callback (GtkWidget *widget, gpointer data);
+static void capplet_close_callback2 (GtkWidget *widget, gpointer data);
 node_data *
 find_node_by_id (gint id)
 {
@@ -53,7 +63,6 @@ exec_capplet (node_data *data)
         gint i;
         gchar *argv[4];
         GList *list;
-
         /* is the silly thing a multi-capplet */
         for (i = 1;data->gde->exec[i];i++) { 
                 if (strstr (data->gde->exec[i], "--cap-id=")) {
@@ -62,13 +71,13 @@ exec_capplet (node_data *data)
                                 /* do multi-capplet stuff... */
                                         data->capplet = CORBA_Object_duplicate (((node_data *)list->data)->capplet, &ev);
                                         capplet_list = g_list_prepend (capplet_list, data);
-
                                         GNOME_capplet_new_multi_capplet(data->capplet,
                                                                         ((node_data *)list->data)->id,
                                                                         data->id,
                                                                         GDK_WINDOW_XWINDOW (data->socket->window),
                                                                         atoi (data->gde->exec[i] + 9),
                                                                         &ev);
+                                        data->state = CAPPLET_ACTIVE;
                                         return; 
                                 }
                         }
@@ -87,27 +96,51 @@ exec_capplet (node_data *data)
         sprintf (temp + 6, "%d",  GDK_WINDOW_XWINDOW (data->socket->window));
         argv[1] = temp;
 
-        temp = g_malloc (sizeof (gchar[7 + strlen(ior)]));
-        sprintf (temp, "--ior=");
-        sprintf (temp + 6, "%s",  ior);
-        argv[2] = temp;
-                
+        argv[2] = NULL;
         /*argv[3] = "--gtk-module=gle";*/
         capplet_list = g_list_prepend (capplet_list, data);
-        /*g_print ("\ngdb %s\nb main\nr %s %s %s\n", data->gde->exec[0], argv[0], argv[1], argv[2]);*/
-        gnome_desktop_entry_launch_with_args (data->gde, 3, argv);
+/*        g_print ("\ngdb %s\nb main\nr %s %s\n", data->gde->exec[0], argv[0], argv[1]);*/
+        
+        gnome_desktop_entry_launch_with_args (data->gde, 2, argv);
+        data->state = CAPPLET_UNREGISTERED;
         g_free (argv[0]);
         g_free (argv[1]);
-        g_free (argv[2]);
 }
 void
-launch_capplet (node_data *data)
+launch_capplet (node_data *data, gboolean exec_new)
 {
         GtkWidget *vbox;
         GtkWidget *separator;
         GtkWidget *bbox;
         GtkWidget *frame;
+        GList *temp;
+        gint page;
+        gboolean show_splash = TRUE; 
         
+        /* If the page has not been changed, nuke it, or tell it to hide itself */
+        if ((current_page > 0) && (data->gde->exec_length)) {
+                page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
+                for (temp = capplet_list; temp; temp = temp->next)
+                        if (temp->data && ((node_data*)temp->data)->notetab_id == page) {
+                                if (((node_data*)temp->data)->id == data->id) {
+                                        return;
+                                } else if (((node_data*)temp->data)->modified == FALSE) {
+                                        show_splash = FALSE;
+                                        if (((node_data*)temp->data)->capplet) {
+                                                GNOME_capplet_cancel (((node_data*)temp->data)->capplet,
+                                                                      ((node_data*)temp->data)->id,
+                                                                      &ev);
+                                                ((node_data*)temp->data)->capplet = NULL;
+                                        }
+                                        close_capplet (show_splash, (node_data*)temp->data);
+                                } else {
+                                        GNOME_capplet_page_hidden (((node_data*)temp->data)->capplet,
+                                                                   ((node_data*)temp->data)->id,
+                                                                   &ev);
+                                }
+                                break;
+                        }
+        }
         /* set up the notebook if needed */
         /* This capplet has not been started yet.  We need to do that. */
         if ((data->id == -1) && (data->gde->exec_length)) {
@@ -115,14 +148,18 @@ launch_capplet (node_data *data)
                         notebook = gtk_notebook_new();
                         gtk_notebook_set_show_tabs (GTK_NOTEBOOK (notebook), FALSE);
                         gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
-                        gtk_widget_ref (splash_screen);
-                        gtk_container_remove (GTK_CONTAINER (container), splash_screen);
+                        /* show we only want to remove the splashscreen if it's showing */
+                        if (show_splash) {
+                                gtk_widget_ref (splash_screen);
+                                gtk_container_remove (GTK_CONTAINER (container), splash_screen);
+                        }
                         gtk_container_add (GTK_CONTAINER (container), notebook);
                         gtk_widget_show (notebook);
                 } 
 
                 vbox = gtk_vbox_new(FALSE, 0);
                 data->socket = gtk_socket_new ();
+                gtk_signal_connect (GTK_OBJECT (data->socket), "destroy", capplet_close_callback, data);
                 separator = gtk_hseparator_new ();
                 bbox = gtk_hbutton_box_new ();
                 gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_SPREAD);
@@ -160,31 +197,47 @@ launch_capplet (node_data *data)
                 gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
                 gtk_box_pack_end (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
                 gtk_box_pack_end (GTK_BOX (vbox), separator, FALSE, FALSE, GNOME_PAD_SMALL);
-                /* this is in case we ever o back to tabs:  We want to keep tabs on this so we can change the text. */
+
+                /* this is in case we ever go back to tabs:
+                 * We want to keep tabs on this so we can change the text. */
                 data->label = gtk_label_new (data->gde->name);
                 gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, data->label);
                 gtk_widget_show_all (vbox);
                 data->notetab_id = current_page++;
                 data->id = id++;
 
-                exec_capplet (data);
-                
+                if (exec_new) {
+                        exec_capplet (data);
+                } else {
+                        capplet_list = g_list_prepend (capplet_list, data);
+                        gtk_ctree_select (data->ctree, data->node);
+                }
         }
-        if (data->notetab_id != -1)
+        if (data->notetab_id != -1) {
                 gtk_notebook_set_page (GTK_NOTEBOOK (notebook), data->notetab_id);
+                if (data->capplet) {
+                        GNOME_capplet_page_shown (data->capplet,
+                                                  data->id,
+                                                  &ev);
+                }
+                gtk_ctree_select (data->ctree, data->node);
+        }
 }
 
 
 void try_button_callback(GtkWidget *widget, gpointer data)
 {
         node_data *nd = (node_data *) data;
+        if  (!nd->capplet)
+                return;
         gtk_widget_set_sensitive (nd->try_button, FALSE);
-
         GNOME_capplet_try (nd->capplet,nd->id, &ev);
 }
 void revert_button_callback(GtkWidget *widget, gpointer data)
 {
         node_data *nd = (node_data *) data;
+        if (!nd->capplet)
+                return;
         GNOME_capplet_revert (nd->capplet,nd->id, &ev);
         gtk_widget_set_sensitive (nd->try_button, FALSE);
         gtk_widget_set_sensitive (nd->revert_button, FALSE);
@@ -195,8 +248,9 @@ void ok_button_callback(GtkWidget *widget, gpointer data)
         node_data *nd = (node_data *) data;
         style = gtk_widget_get_style (GTK_WIDGET (nd->ctree));
 
-        GNOME_capplet_ok (nd->capplet,nd->id, &ev);
-        close_capplet (widget, data);
+        if (nd->capplet)
+                GNOME_capplet_ok (nd->capplet,nd->id, &ev);
+        close_capplet (TRUE, data);
         gtk_ctree_node_set_foreground (nd->ctree, nd->node, &style->fg[GTK_STATE_NORMAL]);
 }
 void cancel_button_callback(GtkWidget *widget, gpointer data)
@@ -205,34 +259,118 @@ void cancel_button_callback(GtkWidget *widget, gpointer data)
         node_data *nd = (node_data *) data;
         style = gtk_widget_get_style (GTK_WIDGET (nd->ctree));
 
-        GNOME_capplet_cancel (nd->capplet,nd->id, &ev);
-        close_capplet (widget, data);
+        if (nd->capplet)
+                GNOME_capplet_cancel (nd->capplet,nd->id, &ev);
+        close_capplet (TRUE, data);
         gtk_ctree_node_set_foreground (nd->ctree, nd->node, &style->fg[GTK_STATE_NORMAL]);
 }
-void close_capplet (GtkWidget *widget, gpointer data)
+static void
+capplet_close_callback (GtkWidget *widget, gpointer data)
+{
+        /* This function tries to clean up correctly. */
+        /* If a capplet dies, it will (try to) shut it down gracefully... */
+        /* It assumes that the capplet was not reparented already */
+        node_data *nd = (node_data *) data;
+        GtkStyle *style;
+
+        if (nd->socket != NULL) {
+                nd->socket = NULL;
+                close_capplet (TRUE, nd);
+                nd->state = CAPPLET_INACTIVE;
+                style = gtk_widget_get_style (GTK_WIDGET (nd->ctree));
+                gtk_ctree_node_set_foreground (nd->ctree, nd->node, &style->fg[GTK_STATE_NORMAL]);
+        }
+}
+static void
+capplet_close_callback2 (GtkWidget *widget, gpointer data)
+{
+        /* This function tries to clean up correctly as well. */
+        /* it has less to clean up though. */
+        /* it needs to stop the timer callback. */
+        timer_entry *te = (timer_entry *) data;
+        if (te->timer_id)
+                gtk_timeout_remove (te->timer_id);
+        g_free (te);
+}
+static gint
+timer_callback (gpointer data)
+{
+        timer_entry *te = (timer_entry *) data;
+
+        /* I don't know if this can happen, but just to be absolutely
+         * safe, we test it. */
+        if ((GTK_SOCKET (te->socket)->plug_window != NULL)
+            || te->count == 0) {
+                gtk_widget_destroy (te->socket);
+                te->timer_id = 0;
+                /* capplet_close_callback2 will handle freeing te */
+                return 0;
+        }
+        te->count--;
+        return 1;
+}
+static void
+queue_socket_destruction (node_data *nd)
+{
+        timer_entry *te;
+
+        te = g_new (timer_entry, 1);
+        te->socket = nd->socket;
+        te->count = 15;
+        te->timer_id = gtk_timeout_add (2000, timer_callback, te);
+        gtk_widget_reparent (nd->socket, hack_widget);
+        gtk_signal_disconnect_by_data (GTK_OBJECT (nd->socket),
+                                       nd);
+        gtk_signal_connect (GTK_OBJECT (nd->socket), "destroy", capplet_close_callback2, te);
+}
+static void
+shutdown_capplet (node_data *nd) {
+       GList *temp;
+
+       if (nd->state == CAPPLET_ACTIVE &&
+           nd->socket && GTK_SOCKET (nd->socket)->plug_window == NULL) {
+               /* Uh oh... We are shutting down a capplet that hasn't realized
+                * yet.  We need to reparent the socket; hide it, and, when it's
+                * realized, have it go away. */
+               /* To do this, we reparent the socket and hide it.
+                * When the plug dies, so goes the socket */
+               queue_socket_destruction (nd);
+       }
+       nd->socket = NULL;
+       gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), nd->notetab_id);
+       nd->id = -1;
+       nd->modified = FALSE;
+       nd->state = CAPPLET_INACTIVE;
+       capplet_list = g_list_remove (capplet_list, nd);
+       if (nd->capplet) {
+               CORBA_Object_release (nd->capplet, &ev);
+               nd->capplet = NULL;
+       }
+       for (temp = capplet_list; temp; temp = temp->next)
+               if (((node_data*)temp->data)->notetab_id > nd->notetab_id)
+                       ((node_data*)temp->data)->notetab_id -=1;
+               
+       nd->notetab_id = -1;
+}
+/* If show_splash is true, then that means that we want to handle whatever is underneath
+ * (ie. putting the splashscreen back, changing the selected node etc. )
+ * Otherwise we ignore what's underneath (ie. we're about to launch another one.) */
+void
+close_capplet (gboolean show_splash, gpointer data)
 {
         node_data *nd = (node_data *) data;
         GList *temp;
 
-        gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), nd->notetab_id);
-        nd->id = -1;
-        nd->modified = FALSE;
-        capplet_list = g_list_remove (capplet_list, nd);
-        if (nd->capplet) {
-                CORBA_Object_release (nd->capplet, &ev);
-                nd->capplet = NULL;
-        }
-        for (temp = capplet_list; temp; temp = temp->next)
-                if (((node_data*)temp->data)->notetab_id > nd->notetab_id)
-                        ((node_data*)temp->data)->notetab_id -=1;
-        nd->notetab_id = -1;
+        shutdown_capplet (nd);
         if (--current_page == 0) {
                 gtk_container_remove (GTK_CONTAINER (container), notebook);
                 notebook = NULL;
-                gtk_container_add (GTK_CONTAINER (container), splash_screen);
-                g_list_remove (capplet_list, nd);
-                gtk_widget_unref (splash_screen);
-        } else {
+                if (show_splash) {
+                        /* if we are only temporarily removing it, we don't show a splash. */
+                        gtk_container_add (GTK_CONTAINER (container), splash_screen);
+                        gtk_widget_unref (splash_screen);
+                }
+        } else if (show_splash) {
                 /* we need to find the current page, and highlight it. */
                 gint page;
 
@@ -246,8 +384,11 @@ void close_capplet (GtkWidget *widget, gpointer data)
                                      ctnode; ctnode = GTK_CTREE_ROW (ctnode)->parent) {
                                         gtk_ctree_expand (((node_data *) temp->data)->ctree,
                                                           ((node_data *) temp->data)->node);
-                                        g_print ("expanding (in theory)\n");
                                 }
+                                GNOME_capplet_page_shown (((node_data*)temp->data)->capplet,
+                                                          ((node_data*)temp->data)->id,
+                                                          &ev);
+
                         }
         }
 }
