@@ -34,12 +34,16 @@ static void server_ok (PortableServer_Servant servant, CORBA_long id, CORBA_Envi
 static void server_cancel (PortableServer_Servant servant, CORBA_long id, CORBA_Environment * ev);
 static void server_help (PortableServer_Servant servant, CORBA_long id, CORBA_Environment * ev);
 static void server_new_multi_capplet(GNOME_capplet _obj, CORBA_long id, CORBA_long newid, CORBA_unsigned_long newxid, CORBA_long newcapid, CORBA_Environment * ev);
+static void server_page_hidden (PortableServer_Servant servant, CORBA_long id, CORBA_Environment * ev);
+static void server_page_shown (PortableServer_Servant servant, CORBA_long id, CORBA_Environment * ev);
 extern void _capplet_widget_server_try(gint id);
 extern void _capplet_widget_server_revert(gint id);
 extern void _capplet_widget_server_ok(gint id);
 extern void _capplet_widget_server_cancel(gint id);
 extern void _capplet_widget_server_help(gint id);
 extern void _capplet_widget_server_new_multi_capplet(gint id, gint capid);
+extern void _capplet_widget_server_page_hidden(gint id);
+extern void _capplet_widget_server_page_shown(gint id);
 
 static struct poptOption cap_options[] = {
         {"id", '\0', POPT_ARG_INT, &id, 0, N_("id of the capplet -- assigned by the control-center"), N_("ID")},
@@ -66,7 +70,9 @@ POA_GNOME_capplet__epv capplet_epv =
         (gpointer)&server_ok,
         (gpointer)&server_cancel,
         (gpointer)&server_help,
-        (gpointer)&server_new_multi_capplet
+        (gpointer)&server_new_multi_capplet,
+        (gpointer)&server_page_hidden,
+        (gpointer)&server_page_shown
 };
 POA_GNOME_capplet__vepv poa_capplet_vepv = { &base_epv, &capplet_epv };
 POA_GNOME_capplet poa_capplet_servant = { NULL, &poa_capplet_vepv };
@@ -124,6 +130,16 @@ server_help (PortableServer_Servant servant, CORBA_long id, CORBA_Environment *
         _capplet_widget_server_help (id);
 }
 static void
+server_page_hidden (PortableServer_Servant servant, CORBA_long id, CORBA_Environment * ev)
+{
+        _capplet_widget_server_page_hidden (id);
+}
+static void
+server_page_shown (PortableServer_Servant servant, CORBA_long id, CORBA_Environment * ev)
+{
+        _capplet_widget_server_page_shown (id);
+}
+static void
 server_new_multi_capplet(GNOME_capplet _obj, CORBA_long id, CORBA_long newid, CORBA_unsigned_long newxid, CORBA_long newcapid, CORBA_Environment * ev)
 {
         GList *temp;
@@ -214,8 +230,11 @@ void *capplet_widget_corba_init(const char *app_id,
                                unsigned int flags,
                                poptContext *return_ctx)
 {
+        GoadServerList *gsdlist = NULL;
         PortableServer_ObjectId objid = {0, sizeof("capplet_interface"), "capplet_interface"};
         PortableServer_POA poa;
+        guint tempxid;
+        gint tempid;
 
         CORBA_exception_init(&ev);
 
@@ -229,10 +248,6 @@ void *capplet_widget_corba_init(const char *app_id,
                 return NULL;
 
         /* sanity check */
-        if ((xid == 0) || (cc_ior == NULL) || (id == -1)) {
-                g_warning ("Insufficient arguments passed to the arg parser.\n");
-                exit (1);
-        }
         if (!orb) {
                 g_warning ("unable to connect to the server.\nexitting...\n");
                 exit (1);
@@ -253,13 +268,34 @@ void *capplet_widget_corba_init(const char *app_id,
         }
         ORBit_custom_run_setup(orb, &ev);
 
+        control_center = goad_server_activate_with_repo_id (NULL,
+                                                            "IDL:GNOME/control_center:1.0",
+                                                            0, NULL);
+        
         /* now we get the control center. */
-        control_center = CORBA_ORB_string_to_object(orb, cc_ior, &ev);
+/*        control_center = CORBA_ORB_string_to_object(orb, cc_ior, &ev);*/
         if (! control_center) {
                 g_warning ("Unable reach the control-center.\nExiting...");
                 exit (1);
         }
-        GNOME_control_center_register_capplet(control_center, id, capplet, &ev);
+        if ((xid == 0) || (id == -1)) {
+                GNOME_control_center_register_capplet_new(control_center,
+                                                          capplet,
+                                                          argv[0],
+                                                          capid,
+                                                          &xid,
+                                                          &id,
+                                                          &ev);
+                /* if it's still 0, then the control-center doesn't wan't us */
+                /* In theory, we should clean up after ourselves at this point */
+                /* but I need to think a bit more about it. */
+                /* Is there any time that we'd want to stay alive w/ CORBA and */
+                /* not be a capplet??? */
+                if ((xid == 0) || (id == -1))
+                        return NULL;
+        }
+        if (GNOME_control_center_register_capplet(control_center, id, capplet, &ev) == -1)
+                return NULL;
         /* we need to get the error value from gnome_CORBA_init */
         return (gpointer) orb;
 }
