@@ -15,33 +15,20 @@ GtkWidget *hpane;
 GtkWidget *container;
 GtkWidget *notebook = NULL;
 GtkWidget *splash_screen;
+GtkWidget *tree;
 GnomeAppBar *status_bar;
+/* We create a random vbox to stuff the socket in when necessary */
+GtkWidget *hack_widget;
 gchar *init_cap = NULL;
-/* Temporary hack to make an RPM.  If anyone ever sees this, please, blow it away */
-#ifndef GNOMEUIINFO_MENU_EXIT_ITEM
-#define GNOMEUIINFO_MENU_EXIT_ITEM(cb, data)                                \
-        { GNOME_APP_UI_ITEM, N_("E_xit"), N_("Exit the program"),           \
-          cb, (gpointer)(data), NULL, GNOME_APP_PIXMAP_STOCK,               \
-	  GNOME_STOCK_MENU_EXIT, GNOME_KEY_NAME_EXIT, GNOME_KEY_MOD_EXIT,   \
-	    NULL }
-
-#define GNOMEUIINFO_MENU_ABOUT_ITEM(cb, data)                               \
-        { GNOME_APP_UI_ITEM, N_("_About..."),                               \
-          N_("About this application"), cb, (gpointer)(data),           \
-	  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,             \
-          0,  (GdkModifierType) 0, NULL }
-#define GNOMEUIINFO_MENU_HELP_TREE(tree) \
-	{ GNOME_APP_UI_SUBTREE, N_("_Help"), NULL, tree, NULL, NULL, \
-		(GnomeUIPixmapType) 0, NULL, 0,	(GdkModifierType) 0, NULL }
-#endif
 
 static GnomeUIInfo mainMenu[] = {
         GNOMEUIINFO_MENU_EXIT_ITEM(exit_callback, NULL), 
         GNOMEUIINFO_END
 };
 static GnomeUIInfo helpMenu[] = {
-	GNOMEUIINFO_HELP("gnomecc"),
 
+        GNOMEUIINFO_ITEM_STOCK(N_("Help"), N_("Help with the GNOME control-center."),
+                               help_callback, GNOME_STOCK_PIXMAP_HELP),
         GNOMEUIINFO_MENU_ABOUT_ITEM(about_callback, NULL),
 
         GNOMEUIINFO_END
@@ -128,17 +115,51 @@ create_exit_dialog (GList *apps)
         
         return retval;
 }
+void
+gtk_widget_set_uposition_internal (GtkWidget *widget,
+                                   gint	     x,
+                                   gint	     y)
+{
+        /* woo hoo -- we love copying code. */
+        /* but we need to set the uposition to -10, -10 */
+  GtkWidgetAuxInfo *aux_info;
+  guint aux_info_key_id = 0;
+
+  g_return_if_fail (widget != NULL);
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+
+  aux_info = g_malloc (sizeof (GtkWidgetAuxInfo));
+  aux_info->x = -1;
+  aux_info->y = -1;
+  aux_info->width = 0;
+  aux_info->height = 0;
+
+  aux_info_key_id = g_quark_from_static_string ("gtk-aux-info");
+  gtk_object_set_data_by_id (GTK_OBJECT (widget), aux_info_key_id, aux_info);
+  
+  aux_info->x = x;
+  aux_info->y = y;
+  
+  if (GTK_WIDGET_REALIZED (widget) && GTK_IS_WINDOW (widget))
+    {
+      gdk_window_set_hints (widget->window, aux_info->x, aux_info->y, 0, 0, 0, 0, GDK_HINT_POS);
+      gdk_window_move (widget->window, aux_info->x, aux_info->y);
+    }
+  
+  if (GTK_WIDGET_VISIBLE (widget) && widget->parent)
+    gtk_widget_size_allocate (widget, &widget->allocation);
+}
 GtkWidget *
 create_window ()
 {
-        GtkWidget *vbox;
         GtkWidget *retval;
-        GtkWidget *tree;
         GdkPixmap *temp_splash;
+        GtkWidget *vbox;
         GtkWidget *sw;
+        GtkWidget *window;
         
         /* create the app */
-        retval = gnome_app_new ("control-center", "Control Center");
+        retval = gnome_app_new ("control-center", _("Control Center"));
 	gtk_signal_connect (GTK_OBJECT (retval), "delete_event",
                             GTK_SIGNAL_FUNC (exit_callback), retval);
 	gnome_app_create_menus (GNOME_APP (retval), parentMenu);
@@ -158,7 +179,8 @@ create_window ()
         gtk_paned_gutter_size (GTK_PANED (hpane), 10);
         tree = generate_tree ();
         sw = gtk_scrolled_window_new (GTK_CLIST (tree)->hadjustment, GTK_CLIST (tree)->vadjustment);
-        gtk_widget_set_usize (sw, 175, 275);
+        /* This is sorta evil */
+        gtk_widget_set_usize (sw, 200, 275);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
         container = gtk_frame_new(NULL);
         gtk_container_set_border_width (GTK_CONTAINER (container), 0);
@@ -179,6 +201,16 @@ create_window ()
         /* and make everyting visible */
         gtk_widget_show_all (retval);
         /*gdk_window_set_back_pixmap (splash_screen->window, temp_splash, FALSE); */
+
+        /* THIS IS AN UGLY HACK */
+        /* We create a window -- override redirect and off screen, to dump
+         * our unused sockets in.  We do this b/c reparenting a hidden socket
+         * causes trouble */
+        window = gtk_window_new (GTK_WINDOW_POPUP);
+        hack_widget = gtk_vbox_new (FALSE, 0);
+        gtk_widget_set_usize (window, 100, 100);
+        gtk_widget_set_uposition_internal (window, -300, -300);
+        gtk_widget_show_all (window);
         return retval;
 }
 
